/*************************************************************************
 *	This code is written by N. Kehtarnavaz and N. Kim as part of the 
 *	textbook "Real-Time Digital Signal Processing Based on TMS320C6000".
 *************************************************************************/

#define CHIP_6713 1

#include <stdio.h>
#include <c6x.h>
#include <csl.h>
#include <csl_mcbsp.h>
#include <csl_irq.h>

#include "dsk6713.h"
#include "dsk6713_aic23.h"

#define N ?? //filter order

/*********************************************************
 *  Declarations
 *********************************************************/
 
// FIR filter coefficients
float b[N+1] = {?,?,?,?}; // insert your FIR filter coefficients here.

int samples[N+1];

DSK6713_AIC23_CodecHandle hCodec;
DSK6713_AIC23_Config config = DSK6713_AIC23_DEFAULTCONFIG; // Codec configuration with default settings

interrupt void serialPortRcvISR(void);

void main()
{
	int	i;

	DSK6713_init();		// Initialize the board support library, must be called first
	hCodec = DSK6713_AIC23_openCodec(0, &config);

	MCBSP_FSETS(SPCR1, RINTM, FRM);
	MCBSP_FSETS(SPCR1, XINTM, FRM);
	MCBSP_FSETS(RCR1, RWDLEN1, 32BIT);
	MCBSP_FSETS(XCR1, XWDLEN1, 32BIT);
	
	DSK6713_AIC23_setFreq(hCodec, DSK6713_AIC23_FREQ_8KHZ);

	for( i = 0 ; i <= N ; i++ )
		samples[i] = 0;
		
	IRQ_globalDisable();			// Globally disables interrupts
	IRQ_nmiEnable();				// Enables the NMI interrupt
	IRQ_map(IRQ_EVT_RINT1,15);		// Maps an event to a physical interrupt
	IRQ_enable(IRQ_EVT_RINT1);		// Enables the event
	IRQ_globalEnable();				// Globally enables interrupts

  	/*************************************************************************
  	 * Main Loop, wait for Interrupt	    								   *					               
  	 ***************************************************************************/

	while(1)
	{
	}
}

interrupt void transmit_ISR()
{
	int temp;
	
	unsigned int output;	
	int i;
	int yn=0;

	temp = MCBSP_read(DSK6713_AIC23_DATAHANDLE); // read the input sample
	// main FIR filtering computation part
	// Update input
	
	for(i=N;i>0;i--)
		samples[i]=samples[i-1]; // move the samples down to make space for the current (newest) sample
	
	samples[0]=temp; // samples always keeps the current sample x[n].

	// Filtering	
	yn=0; // result
	for(i=0;i<N+1;i++)
		yn+= samples[i] * h[i]; // convolution sum


	output = (yn & 0xffff0000); // we use the right channel

	MCBSP_write(DSK6713_AIC23_DATAHANDLE, output );
}
